/*
 * Copyright 2003, 2004 Berend "Kirk" Wouda
 *
 * This file is part of JavaDoctor.
 *
 * JavaDoctor is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JavaDoctor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JavaDoctor; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// The package of this class... Naturally.
package gui;

// Import the GUI classes. We are not 1337 enough to use Swing yet. And we probably
// don't need to be for this anyways.
import java.awt.*;

// Import the controlling class.
import system.JDControl;

// Import my gui classes.
import kirk.gui.closer.*;
import kirk.gui.dialog.*;

/**
 * This class is the GUI for the JavaDoctor program. It interacts with the user,
 * of course, and JDControl, which does the internal stuff. That would be the
 * conversion etc.
 *
 * @author Berend "Kirk" Wouda
 * @version 1.02
 * @since 1.00
 */
public class JDFrame extends Frame {
	/**
	 * <p>Construct a new JDFrame to have the user stare at in awe. Perhaps.
	 * Sets the title to a standard funky thing. Or maybe not so funky.<p>
	 *
	 * <p>Furthermore it initialises a lot of stuff. Like components...
	 * action commands... yeah... that stuff.</p>
	 */
	public JDFrame() {
		// Set the frame title to JavaDoctor. Which happens to be the name of this
		// program! So the user can recognize it! Ooh!
		super("JavaDoctor v1.02");

		// Set the LayOutManager to... GridBag... tempting... but hard...
		// No wait, I'll make it GridLayout instead. Then I can easier customize the
		// 6 grid panels. Which are actually homemade Panels! Whaah! XD
		setLayout(new GridLayout(2, 3));

		// Make a new control class and associate it with this GUI.
		control = new JDControl(this);

		// Make a new WindowCloser as the WindowListener for this Frame.
		// We want it to end the program as well.
		addWindowListener(new WindowCloser(true));

		// Set the screensize.
		setSize(600, 400);

		// Set the location (in the middle of the screen).
		Dimension screensize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((screensize.width - getWidth()) / 2, (screensize.height - getHeight())/ 2);

		// Construct the panels. Feed them control as ActionListener.
		folderpanel = new FolderPanel();
		buttonpanel = new ButtonPanel(control);
		listpanel = new ListPanel(control);
		classpanel = new TagPanel("Class", control);
		methodpanel = new TagPanel("Method", control);
		attributepanel = new TagPanel("Attribute", control);

		// Add the Panels to the Grid.
		add(folderpanel);
		add(buttonpanel);
		add(listpanel);
		add(classpanel);
		add(methodpanel);
		add(attributepanel);
	}


	/**
	 * Shows a Dialog with the passed message.
	 *
	 * @param message the passed message.
	 */
	public void popup(String message) {
		// Create a Dialog that is going to show the message.
		ErrorDialog popup = new ErrorDialog(this, message);

		// Show this dialog (that will also block this method until it is closed).
		popup.show();
	}

	/**
	 * Returns the String entered by the user a popped up dialog box.
	 *
	 * @param message The message asking for a tag.
	 * @return The String entered by the user a popped up dialog box, or
	 * <code>null</code> if the user canceled.
	 */
	public String getPopupText(String message) {
		// Ask for a value from an input retriever and return it.
		return SingleInputRetriever.getValue(this, message, 20);
	}


	/**
	 * Returns the Input Folder that the user has chosen.
	 *
	 * @return The Input Folder given by the user.
	 */
	public String getInputFolder() {
		// Retreive the contents of the inputFolder field from the folder panel,
		// and return that.
		return folderpanel.getInputFolder();
	}

	/**
	 * Returns the Output Folder that the user has chosen.
	 *
	 * @return The Output Folder given by the user.
	 */
	public String getOutputFolder() {
		// Retreive the contents of the outputfolder field from the folder panel,
		// and return that.
		return folderpanel.getOutputFolder();
	}

	/**
	 * Returns whether the user wants to have the Output Folder the same as the
	 * Input Folder.
	 *
	 * @return Whether the user wants to have the Output Folder the same as the
	 * Input Folder.
	 */
	public boolean outputFolderIsInputFolder() {
		// Retreive the value from the folder panel and return that.
		return folderpanel.outputFolderIsInputFolder();
	}


	/**
	 * Returns a list of the selected filenames.
	 *
	 * @return A list of the selected filenames.
	 */
	public String[] getFileList() {
		// Retrieve the list of selected filenames from the list panel, and return
		// that.
		return listpanel.getFileList();
	}

	/**
	 * Set the list of the filenames to the passed filenames.
	 *
	 * @param filelist A list of filenames.
	 */
	public void setFileList(String[] filelist) {
		// Set the list of filenames in the list panel.
		listpanel.setFileList(filelist);
	}


	/**
	 * Returns whether the star insets style of commentins should be used.
	 *
	 * @return Whether the star insets style of commentins should be used.
	 */
	public boolean getStarInsets() {
		// Retrieve the star inset setting from the button panel, and return that.
		return buttonpanel.getStarInsets();
	}


	/**
	 * Adds a class tag to the classtag list.
	 *
	 * @param classtag The classtag to be added.
	 */
	public void addClassTag(String classtag) {
		// Tell the class tag panel to add the tag.
		classpanel.addTag(classtag);
	}

	/**
	 * Adds a class tag to the classtag list on the passed index.
	 *
	 * @param classtag The classtag to be added.
	 * @param index The index to add the passed classtag on.
	 */
	public void addClassTag(String classtag, int index) {
		// Tell the class tag panel to add the tag.
		classpanel.addTag(classtag, index);
	}

	/**
	 * Removes a class tag from the classtag list.
	 *
	 * @param index The index of the class tag to be removed.
	 */
	public void removeClassTag(int index) {
		// Tell the class tag panel to remove the tag.
		classpanel.removeTag(index);
	}

	/**
	 * Returns the amount of class tags.
	 *
	 * @return The amount of class tags.
	 */
	public int getNumberOfClassTags() {
		// Retreive the amount of tags from the class panel, and return that.
		return classpanel.getNumberOfTags();
	}

	/**
	 * Returns the index of the currently selected class tag.
	 *
	 * @return The index of the currently selected class tag.
	 */
	public int getSelectedClassTagIndex() {
		// Retrieve it from the class tag panel, and return it.
		return classpanel.getSelectedTagIndex();
	}

	/**
	 * Returns the currently selected class tag.
	 *
	 * @return The currently selected class tag.
	 */
	public String getSelectedClassTag() {
		// Retrieve it from the class tag panel, and return it.
		return classpanel.getSelectedTag();
	}

	/**
	 * Selects the class tag at the passed index.
	 *
	 * @param index The index of the class tag to be added.
	 */
	public void selectClassTag(int index) {
		// Tell the class tag panel to select the tag.
		classpanel.selectTag(index);
	}

	/**
	 * Returns the list of tags the user wants to use in the class comments.
	 *
	 * @return A list of <code>String</code>s that represent tags.
	 */
	public String[] getClassTags() {
		// Retrieve the tags from the class tag panel, and return that.
		return classpanel.getTags();
	}

	/**
	 * Returns whether the user wants class comments to be converted.
	 *
	 * @return Whether the user wants class comments to be converted.
	 */
	public boolean getConvertClassComment() {
		// Retrieve it from the class tag panel, and return that.
		return classpanel.getConvertComment();
	}

	/**
	 * Returns whether the user wants comments to be added to their classes.
	 *
	 * @return Whether the user wants comments to be added to their classes.
	 */
	public boolean getAddClassComment() {
		// Retrieve it from the class tag panel, and return that.
		return classpanel.getAddComment();
	}

	/**
	 * Returns the amount of white lines the user wants to be added before a class.
	 *
	 * @return The amount of white lines the user wants to be added (as a
	 * <code>String</code>).
	 */
	public String getClassWhiteLines() {
		// Retrieve it from the class tag panel, and return that.
		return classpanel.getWhiteLines();
	}


	/**
	 * Adds a method tag to the methodtag list.
	 *
	 * @param methodtag The methodtag to be added.
	 */
	public void addMethodTag(String methodtag) {
		// Tell the method tag panel to add the tag.
		methodpanel.addTag(methodtag);
	}

	/**
	 * Adds a method tag to the methodtag list on the passed index.
	 *
	 * @param methodtag The methodtag to be added.
	 * @param index The index to add the passed methodtag on.
	 */
	public void addMethodTag(String methodtag, int index) {
		// Tell the method tag panel to add the tag.
		methodpanel.addTag(methodtag, index);
	}

	/**
	 * Removes a method tag from the methodtag list.
	 *
	 * @param index The index of the methodtag to be removed.
	 */
	public void removeMethodTag(int index) {
		// Tell the method tag panel to remove the tag.
		methodpanel.removeTag(index);
	}

	/**
	 * Returns the amount of method tags.
	 *
	 * @return The amount of method tags.
	 */
	public int getNumberOfMethodTags() {
		// Retreive the amount of tags from the method panel, and return that.
		return methodpanel.getNumberOfTags();
	}

	/**
	 * Returns the index of the currently selected method tag.
	 *
	 * @return The index of the currently selected method tag.
	 */
	public int getSelectedMethodTagIndex() {
		// Retrieve it from the method tag panel, and return it.
		return methodpanel.getSelectedTagIndex();
	}

	/**
	 * Returns the currently selected method tag.
	 *
	 * @return The currently selected method tag.
	 */
	public String getSelectedMethodTag() {
		// Retrieve it from the method tag panel, and return it.
		return methodpanel.getSelectedTag();
	}

	/**
	 * Selects the method tag at the passed index.
	 *
	 * @param index The index of the method tag to be added.
	 */
	public void selectMethodTag(int index) {
		// Tell the method tag panel to select the tag.
		methodpanel.selectTag(index);
	}

	/**
	 * Returns the list of tags the user wants to use in the method comments.
	 *
	 * @return A list of <code>String</code>s that represent tags.
	 */
	public String[] getMethodTags() {
		// Retrieve the tags from the method tag panel, and return that.
		return methodpanel.getTags();
	}

	/**
	 * Returns whether the user wants method comments to be converted.
	 *
	 * @return Whether the user wants method comments to be converted.
	 */
	public boolean getConvertMethodComment() {
		// Retrieve it from the method tag panel, and return that.
		return methodpanel.getConvertComment();
	}

	/**
	 * Returns whether the user wants comments to be added to their methods.
	 *
	 * @return Whether the user wants comments to be added to their methods.
	 */
	public boolean getAddMethodComment() {
		// Retrieve it from the method tag panel, and return that.
		return methodpanel.getAddComment();
	}

	/**
	 * Returns the amount of white lines the user wants to be added before a method.
	 *
	 * @return The amount of white lines the user wants to be added (as a
	 * <code>String</code>).
	 */
	public String getMethodWhiteLines() {
		// Retrieve it from the method tag panel, and return that.
		return methodpanel.getWhiteLines();
	}


	/**
	 * Adds an attribute tag to the attributetag list.
	 *
	 * @param attributetag The attributetag to be added.
	 */
	public void addAttributeTag(String attributetag) {
		// Tell the attribute tag panel to add the tag.
		attributepanel.addTag(attributetag);
	}

	/**
	 * Adds an attribute tag to the attributetag list on the passed index.
	 *
	 * @param attributetag The attributetag to be added.
	 * @param index The index to add the passed attributetag on.
	 */
	public void addAttributeTag(String attributetag, int index) {
		// Tell the attribute tag panel to add the tag.
		attributepanel.addTag(attributetag, index);
	}

	/**
	 * Removes an attribute tag from the attributetag list.
	 *
	 * @param index The index of the attributetag to be removed.
	 */
	public void removeAttributeTag(int index) {
		// Tell the attribute tag panel to remove the tag.
		attributepanel.removeTag(index);
	}

	/**
	 * Returns the amount of attribute tags.
	 *
	 * @return The amount of attribute tags.
	 */
	public int getNumberOfAttributeTags() {
		// Retreive the amount of tags from the attribute panel, and return that.
		return attributepanel.getNumberOfTags();
	}

	/**
	 * Returns the index of the currently selected attribute tag.
	 *
	 * @return The index of the currently selected attribute tag.
	 */
	public int getSelectedAttributeTagIndex() {
		// Retrieve it from the attribute tag panel, and return it.
		return attributepanel.getSelectedTagIndex();
	}

	/**
	 * Returns the currently selected attribute tag.
	 *
	 * @return The currently selected attribute tag.
	 */
	public String getSelectedAttributeTag() {
		// Retrieve it from the attribute tag panel, and return it.
		return attributepanel.getSelectedTag();
	}

	/**
	 * Selects the attribute tag at the passed index.
	 *
	 * @param index The index of the attribute tag to be added.
	 */
	public void selectAttributeTag(int index) {
		// Tell the attribute tag panel to select the tag.
		attributepanel.selectTag(index);
	}

	/**
	 * Returns the list of tags the user wants to use in the attribute comments.
	 *
	 * @return A list of <code>String</code>s that represent tags.
	 */
	public String[] getAttributeTags() {
		// Retrieve the tags from the attribute tag panel, and return that.
		return attributepanel.getTags();
	}

	/**
	 * Returns whether the user wants attribute comments to be converted.
	 *
	 * @return Whether the user wants attribute comments to be converted.
	 */
	public boolean getConvertAttributeComment() {
		// Retrieve it from the attribute tag panel, and return that.
		return attributepanel.getConvertComment();
	}

	/**
	 * Returns whether the user wants comments to be added to their attributes.
	 *
	 * @return Whether the user wants comments to be added to their attributes.
	 */
	public boolean getAddAttributeComment() {
		// Retrieve it from the attribute tag panel, and return that.
		return attributepanel.getAddComment();
	}

	/**
	 * Returns the amount of white lines the user wants to be added before an
	 * attribute.
	 *
	 * @return The amount of white lines the user wants to be added (as a
	 * <code>String</code>).
	 */
	public String getAttributeWhiteLines() {
		// Retrieve it from the attribute tag panel, and return that.
		return attributepanel.getWhiteLines();
	}

	/**
	 * Selects all the files in the list of files.
	 */
	public void selectAllFiles() {
		// Tell the list panel to do that.
		listpanel.selectAllFiles();
	}

	/**
	 * Deselects all the files in the list of files.
	 */
	public void deselectAllFiles() {
		// Tell the list panel to do that.
		listpanel.deselectAllFiles();
	}


	/**
	 * The FolderPanel for the folders.
	 */
	protected FolderPanel folderpanel;

	/**
	 * The ButonPanel for the buttons.
	 */
	protected ButtonPanel buttonpanel;

	/**
	 * The ListPanel for the list.
	 */
	protected ListPanel listpanel;

	/**
	 * The TagPanel for the class tags.
	 */
	protected TagPanel classpanel;

	/**
	 * The TagPanel for the method tags.
	 */
	protected TagPanel methodpanel;

	/**
	 * The TagPanel for the attribute tags.
	 */
	protected TagPanel attributepanel;


	/**
	 * The control class for this GUI. It does all the work, you could say.
	 */
	protected JDControl control;
}