/*
 * Copyright 2003, 2004 Berend "Kirk" Wouda
 *
 * This file is part of JavaDoctor.
 *
 * JavaDoctor is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JavaDoctor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JavaDoctor; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// Tee paakaach.
package system;

// Import the IOException class.
import java.io.IOException;

// Import the class of all classes... Wait, no, that's ArrayList...
import java.util.StringTokenizer;

// Import the class of all classes. Turns out I need it... again! Woo!
import java.util.ArrayList;

// Import my utility classes.
import kirk.io.*;

/**
 * This class opens up the given file and converts the comments when you call the
 * various methods. It can also add comments where there are not comments yet.
 * After you made all your changes, call save() to write them to the file.
 *
 * @author Berend "Kirk" Wouda
 * @version 1.02
 * @since 1.00
 */
public class CommentConverter {
	/**
	 * Constructs a new CommentConverter over filename, and sets the appropriate
	 * javadoc style.
	 *
	 * @param filename The name of the file to be opened.
	 * @param starinsets The javadoc style. true when the star insets should be used,
	 * false for no star insets.
	 * @throws IOException When an IO error is encountered.
	 */
	public CommentConverter(String filename, boolean starinsets) throws IOException {
		// Construct a FileLineList around filename. Make it so that it can write.
		file = new FileLineList(filename, true);

		// Set the javadoc comment style.
		this.starinsets = starinsets;
	}


	/**
	 * Saves the changes this class made to the file.
	 *
	 * @param filename The name of the file to save to.
	 */
	public void save(String filename) throws IOException {
		// Catch errors.
		try {
			// Flush the changes to FileLineList to filename.
			file.flush(filename);
		}
		// Catch WriteNotAllowedException...
		catch(WriteNotAllowedException e) {
			// This should NOT happen. So...
		}
	}


	/**
	 * Converts all the non-javadoc class comments in this file to javadoc comments
	 * using the set style. Also it puts in any class tags that are requested.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all class comments.
	 */
	public void convertClassComments(String[] tags) {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is a class line.
			// If it is, the line before this needs to contain one or more
			// instances of non-javadoc comment, which will then be
			// converted. We leave that to a special method.
			if(isClass(index))  replaceComment(index, tags);
		}
	}

	/**
	 * Adds all the javadoc class comments to all classes without comment in this
	 * file using the set style. Also it puts in any class tags that are requested,
	 * as well as the requested amount of white lines.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all class comments.
	 * @param whitelines The number of white lines that has to be in each class
	 * comment.
	 */
	public void addClassComments(String[] tags, int whitelines) {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is a class line.
			// If it is, the line before this needs to contain no
			// instances of non-javadoc comment, which will then be
			// added. We leave that to a special method.
			if(isClass(index))  addComment(index, tags, whitelines);
		}
	}


	/**
	 * Converts all the non-javadoc method comments in this file to javadoc comments
	 * using the set style. Also it puts in any method tags that are requested.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all method comments.
	 * @throws Exception When a file format error is encountered.
	 */
	public void convertMethodComments(String[] tags) throws Exception {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is a method line.
			// If it is, the line before this needs to contain one or more
			// instances of non-javadoc comment, which will then be
			// converted. We leave that to a special method.
			// Also, convert any tags that have method stuff in it, like
			// @param, @return and @throws.
			if(isMethod(index)) replaceComment(index, convertMethodTags(index, tags));
		}
	}

	/**
	 * Adds all the javadoc method comments to all methods without comment in this
	 * file using the set style. Also it puts in any method tags that are requested,
	 * as well as the requested amount of white lines.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all method comments.
	 * @param whitelines The number of white lines that has to be in each method
	 * comment.
	 * @throws Exception When a file format error is encountered.
	 */
	public void addMethodComments(String[] tags, int whitelines) throws Exception {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is a method line.
			// If it is, the line before this needs to contain no
			// instances of non-javadoc comment, which will then be
			// added. We leave that to a special method.
			if(isMethod(index))  addComment(index, convertMethodTags(index, tags), whitelines);
		}
	}


	/**
	 * Converts all the non-javadoc attribute comments in this file to javadoc
	 * comments using the set style. Also it puts in any attribute tags that are
	 * requested.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all attribute comments.
	 */
	public void convertAttributeComments(String[] tags) {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is an attribute line.
			// If it is, the line before this needs to contain one or more
			// instances of non-javadoc comment, which will then be
			// converted. We leave that to a special method.
			if(isAttribute(index))  replaceComment(index, tags);
		}
	}

	/**
	 * Adds all the javadoc attribute comments to all attributes without comment in
	 * this file using the set style. Also it puts in any attribute tags that are
	 * requested, as well as the requested amount of white lines.
	 *
	 * @param tags An array of Strings. Each String is a javadoc tag or something
	 * that the user wanted in all attribute comments.
	 * @param whitelines The number of white lines that has to be in each attribute
	 * comment.
	 */
	public void addAttributeComments(String[] tags, int whitelines) {
		// Go through the file...
		for(int index = 0; index < file.getNumberOfLines(); index++) {
			// ...and per line check whether it is an attribute line.
			// If it is, the line before this needs to contain no
			// instances of non-javadoc comment, which will then be
			// added. We leave that to a special method.
			if(isAttribute(index))  addComment(index, tags, whitelines);
		}
	}


	/**
	 * Goes through tags and convert any special javadoc tags that have to with the
	 * method signature at index. When this method is called, it is assumed that
	 * there is a method there.
	 *
	 * @param index The index of the method signature where the info comes from.
	 * @param tags The tags to be converted.
	 * @return The converted tags.
	 * @throws Exception When a file format error is encountered.
	 */
	protected String[] convertMethodTags(int index, String[] tags) throws Exception {
		// The new tag "array" that will be returned.
		ArrayList newtags = new ArrayList();

		// The method signature at index:
		String method = file.get(index);

		// Go through tags.
		for(int count = 0; count < tags.length; count++) {
			// The current tag:
			String tag = tags[count];

			// The first appearance of @param.
			int param = tag.indexOf("@param");

			// The first appearance of @return.
			int ret = tag.indexOf("@return");

			// The first appearance of @throws.
			int thr = tag.indexOf("@throws");

			// Check the current tag.
			if(param >= 0) {
				// @param is in there. We need to insert all the parameters of the
				// method at index as javadoc @param comment at that position.
				// Find the first parameter. It should be the second token after the
				// first occurence of "(".
				// Construct a StringTokenizer over the String between () in the
				// method signature. Also tokenize on commas, so that only types and
				// parameters remain.
				StringTokenizer tokenizer = new StringTokenizer(method.substring(method.indexOf("(") + 1, method.indexOf(")")), " ,\t\n\r\f");

				// Go through the parameters.
				while(tokenizer.hasMoreTokens()) {
					// Each second token is the parameter.
					// Skip the first token.
					tokenizer.nextToken();

					// Construct a StringBuffer around the current tag, and add the
					// current token to the current tag, at the point where @param
					// appeared. @param will be replaced by "@param parameter".
					// Turn this into a String, and add it to the list of new tags.
					newtags.add(new StringBuffer(tag).replace(param, param + 6, "@param " + tokenizer.nextToken()).toString());
				}
			}
			else  if(ret >= 0) {
				// @return is in there. Fortunately there is only one return type,
				// but it can also be void, which means there is no return type.
				// Of course, constructors don't have return types.
				// Construct a StringTokenizer over the String, from the beginning
				// to the point (not including) where "(" appears.
				StringTokenizer tokenizer = new StringTokenizer(method.substring(0, method.indexOf("(") - 1));

				// Go through the tokens. The second to last token should be the
				// return type.
				while(tokenizer.hasMoreTokens()) {
					// Check if the next token is second to last token.
					// If it is, and stop the loop.
					if(tokenizer.countTokens() == 2)  break;
					// If it isn't, go on to the next token.
					else  tokenizer.nextToken();
				}

				// Set the returntype as the next token.
				String returntype = tokenizer.nextToken();

				// Check if the returntype is not "void", "public", "protected" or
				// "private". If it is void, then nothing has to be added to the tag
				// list.
				if(!returntype.equals("void") && !returntype.equals("public") && !returntype.equals("protected") && !returntype.equals("private")) {
					// Add the returntype to the list, javadoc style.
					// Since the returntype doens't appear in the @return tag, we can
					// just add the current tag.
					newtags.add(tag);
				}
			}
			else  if(thr >= 0) {
				// A method can throw any number of exceptions. They are listed after
				// the reserved word "throws" and go on, comma seperated, until the
				// end of the method signature. This is either the end of the line,
				// the start of the method ("{"), or comment at the end of the line.
				// The StringTokenizer that will help us retrieve the exceptions.
				StringTokenizer tokenizer;

				// The occurances:
				int accolade = method.indexOf("{");
				int comment = method.indexOf("//");
				int exceptions = method.indexOf("throws") + 7;

				// Check whether "throws" is in the methods signature (adjust for
				// the already added value).
				if(exceptions >= 7) {
					// Check whether "{" or "//" is in the method signature (Ok, ok, so
					// it's technically not a part of it. Shut up :^P).
					if(accolade < 0 && comment < 0) {
						// Construct a StringTokenizer over the part of the String
						// starting at the first occurance of "throws" to the end of the
						// String.
						tokenizer = new StringTokenizer(method.substring(exceptions), " ,\t\n\r\f");
					}
					// "{" and/or "//" are in there.
					// Check whether they both are.
					else if(accolade >= 0 && comment >= 0) {
						// Construct a StringTokenizer over the part of the String
						// starting at the first occurance of "throws" to the first
						// occurance of "{" or "//" (not including), whichever is
						// smaller.
						tokenizer = new StringTokenizer(method.substring(exceptions), " ,\t\n\r\f");
					}
					// They aren't. Check whether it is "{".
					else if(accolade >= 0) {
						// Construct a StringTokenizer over the part of the String
						// starting at the first occurance of "throws" to the first
						// occurance of "{" (not including).
						tokenizer = new StringTokenizer(method.substring(exceptions, accolade), " ,\t\n\r\f");
					}
					// It isn't. It must be "//".
					else {
						// Construct a StringTokenizer over the part of the String
						// starting at the first occurance of "throws" to the first
						// occurance of "//" (not including).
						tokenizer = new StringTokenizer(method.substring(exceptions, comment), " ,\t\n\r\f");
					}

					// Go though the tokens. Each token is an exception, which will be
					// added to the newtags list.
					while(tokenizer.hasMoreTokens()) {
						// Put the current token into tag and add that to the list.
						// To achieve this, we construct a StringBuffer around tag and we
						// replace @throws with "@throws exception". Then we turn that
						// into a String and add it to the list.
						newtags.add(new StringBuffer(tag).replace(thr, thr + 7, "@throws " + tokenizer.nextToken()).toString());
					}
				}
			}
			// There is no special javadoc stuff in this tag.
			else {
				// Add the current tag to the list of new tags.
				newtags.add(tags[count]);
			}
		}

		// Return the new list with tags as an array of Strings.
		return (String[]) newtags.toArray(new String[0]);
	}

	/**
	 * <p>Returns whether the line indicated by index is a class line.
	 * Examples of class lines:<p>
	 *
	 * <p><ul>
	 * <li><code>public class Moo {</code></li>
	 * <li><code>private class Baa extends Nothing {</code></li>
	 * </ul></p>
	 *
	 * <p>etc.</p>
	 *
	 * @param index The line that needs to be checked.
	 * @return Whether the line indicated by index is a class line.
	 */
	protected boolean isClass(int index) {
		// Check whether the first token is public, protected or private.
		if(isToken(index, 0, "public", true) || isToken(index, 0, "protected", true) || isToken(index, 0, "private", true)) {
			// Return whether the second token is class or interface. Because if it
			// isn't, this method fails. If it is, this method succeeds.
			return  isToken(index, 1, "class", true) || isToken(index, 1, "interface", true);
		}
		// This line is not a class line. Indicate this by returning false.
		else  return false;
	}

	/**
	 * <p>Returns whether the line indicated by index is a method line.
	 * Examples of method lines:</p>
	 *
	 * <p><ul>
	 * <li><code>public void baabaa(int moo);</code></li>
	 * <li><code>public static final Fart getFartFromFartCollection(int index) throws
	 * FartNotFoundException {</code></li>
	 * </ul></p>
	 *
	 * <p>etc.</p>
	 *
	 * @param index The line that needs to be checked.
	 * @return Whether the line indicated by index is a method line.
	 */
	protected boolean isMethod(int index) {
		// Check whether the line isn't a class.
		if(!isClass(index)) {
			// Check whether the first token is public, protected or private.
			if(isToken(index, 0, "public", true) || isToken(index, 0, "protected", true) || isToken(index, 0, "private", true)) {
				// Check whether the line contains "(" (method parameter list
				// opening) before any instance of "//" (single line comment)
				// "{" (block start), " (initialization String that can be present
				// in attributes) and "=" (assignment symbol that can be present in
				// attributes.
				// Retrieve the line.
				String line = file.get(index);

				// Go through line.
				for(int count = 0; count < line.length(); count++) {
					// Check the current character and select the correct action:
					switch(line.charAt(count)) {
						// If we encounter a single line comment marker then this
						// line is not a method line.
						case '/': return false;

						// If we encounter a block starting tag, then this line is
						// not a method line.
						case '{': return false;

						// If we encounter a double quote, this line is an attribute
						// line (so not a method line, if you get my drift :^P).
						case '"': return false;

						// If we encounter an equals sign, this line is an attribute
						// line (so not a method line, if you get my drift :^P).
						case '=': return false;

						// If we encounter the method parameter opening tag this line
						// is a method line.
						case '(': return true;
					}
				}

				// If we get here, then this line is certainly not a method line.
				return false;
			}
			// This line is not a method line. Indicate this by returning false.
			else  return false;
		}
		// This line is not an method line. Indicate this by returning false.
		else  return false;
	}

	/**
	 * <p>Returns whether the line indicated by index is an attribute line.
	 * Examples of attribute lines:</p>
	 *
	 * <p><ul>
	 * <li><code>public final String filetodestroy =
	 * "C:\Windows\win.ini";</code></li>
	 * <li><code>private transient int badattr;</code></li>
	 * <ul></p>
	 *
	 * <p>etc.</p>
	 *
	 * @param index The line that needs to be checked.
	 * @return Whether the line indicated by index is an attribute line.
	 */
	protected boolean isAttribute(int index) {
		// Check whether the line isn't a class or method.
		if(!isClass(index) && !isMethod(index)) {
			// Return true if the first token is public, protected or private, or
			// false otherwise.
			return isToken(index, 0, "public", true) || isToken(index, 0, "protected", true) || isToken(index, 0, "private", true);
		}
		// This line is not an attribute line. Indicate this by returning false.
		else  return false;
	}


	/**
	 * <p>Returns whether the token on tokenindex in the String at index in file, is
	 * equal to the passed string, when tokenized on whitespace.</p>
	 *
	 * <p>A bit cheating, but if you specify a negative number as an index then any
	 * token is allowed to match.</p>
	 *
	 * @param fileindex The index of the String to check.
	 * @param tokenindex The index of the token to check.
	 * @param string The string to be checked for.
	 * @param precise Whether the matching has to be precise, or it has to starts
	 * with string.
	 * @return Whether the token is equal to <code>string</code>. false in any case if
	 * tokenindex is larger than the amount of tokens in the String, or when
	 * <code>fileindex</code> is out of bounds.
	 *
	 */
	protected boolean isToken(int fileindex, int tokenindex, String string, boolean precise) {
		// Check whether the fileindex is in bounds.
		if(fileindex >= 0 && fileindex < file.getNumberOfLines()) {
			// We use StringTokenizer to filter out all the whitespace.
			StringTokenizer tokenizer = new StringTokenizer(file.get(fileindex));

			// The index of the current token:
			int index = 0;

			// Go through the tokens.
			while(tokenizer.hasMoreTokens()) {
				// Retrieve the current token.
				String token = tokenizer.nextToken();

				// Check to see if tokenindex is negative (any match allowed) or if
				// the current index is the same as the required index.
				if(tokenindex < 0 || index == tokenindex) {
					// Check whether the matching needs to be precise
					if(precise) {
						// The token needs to be equal to the passed String. If it
						// is, return true. If it isn't, we go on.
						if(token.equals(string))  return true;
					}
					else {
						// The token needs to start with the passed String. If it
						// does, return true. If it isn't, we go on.
						if(token.startsWith(string))  return true;
					}
				}

				// Increase the index.
				index++;
			}

			// If we get here either the index was too large, or the String just
			// didn't fit. In both cases, the same thing is returned. You guessed it.
			return false;
		}
		// fileindex is out of bounds. Return false.
		else  return false;
	}


	/**
	 * Replaces the comment above index with the set javadoc style comment.
	 * Note that white lines in comment that isn't multiline comment will break off
	 * the comment converting. This is because it's too much trouble to distinguish
	 * them from class/method/attribute seperating whitelines, which we like to keep.
	 * They probably won't happen much anyways.
	 *
	 * @param index The index of the method/class/attribute that needs comment
	 * converted.
	 * @param tags The tags to be inserted.
	 */
	protected void replaceComment(int index, String[] tags) {
		// Catch errors.
		try {
			// Get the indentation for the comment.
			String indent = getIndentation(index);

			// Construct a new ArrayList to put the comments in.
			ArrayList commentlist = new ArrayList();

			// Decrease the index to have it point at the comment closest to the
			// class/method/attribute.
			index--;

			// Go through the lines above the one indicated by index.
			// If they are comments, proceed. Add comments to commentlist.
			// White lines that aren't part of multiline comment will not show up!
			// Go until we encounter a line that is not comment, or the start of the
			// file is reached. Of course, the index may not exceed the number of
			// lines in the file.
			while(index >= 0 && index < file.getNumberOfLines()) {
				// We use StringTokenizer to filter out all the whitespace.
				StringTokenizer tokenizer = new StringTokenizer(file.get(index));

				// Check whether there are tokens.
				if(tokenizer.hasMoreTokens()) {
					// Retrieve the current token.
					String token = tokenizer.nextToken();

					// Check whether the line is comment.
					if(token.startsWith("//")) {
						// If it is, add it to the list (filter out the // and any
						// leading and trailing whitespace though).
						String comment = file.get(index);
						commentlist.add(comment.substring(comment.indexOf("//") + 2).trim());

						// Then, remove this line from the list.
						// Remove the line.
						file.remove(index);
					}
					// It it isn't, then there is no more comment. So we stop.
					else  break;
				}
				// There is only whitespace on this line. There should be no more
				// comment. So we stop.
				else  break;

				// Decrease the index.
				index--;
			}

			// So now we got an ArrayList full of Strings. These Strings are the comments
			// of the class/method/attribute on the line indicated by index, in reverse
			// order. So, we go through the list, and insert each line into the file with
			// the set javadoc style of commenting. Then, we put in a white line, and
			// then the tags. All in reverse order. Why? That way we don't need to change
			// index. Except we need to increase index once, so that it points to the
			// position the lines need to be inserted. Man, I feel like I'm writing in C.
			index++;

			// Of course, we only have to do this if there was any comment (we're
			// converting, after all).
			if(commentlist.size() > 0) {
				// Check which javadoc style is required.
				if(starinsets) {
					// First, we insert the javadoc comment closing tag.
					file.insert(indent + " */", index);

					// Then, we insert the list of tags.
					// Go through the array, backwards.
					for(int count = tags.length - 1; count >= 0; count--) {
						// Retrieve the tag and insert it into file, with the javadoc
						// star in front of it.
						file.insert(indent + " * " + tags[count].trim(), index);
					}

					// Then, we insert a "white line", in this case a star (if there were
					// any tags, that is).
					if(tags.length > 0)  file.insert(indent + " * ", index);

					// Then, we insert the comments.
					// Go through commentlist.
					for(int count = 0; count < commentlist.size(); count++) {
						// Retrieve the comment from commentlist, and insert it into
						// file, with the javadoc star in front of it.
						file.insert(indent + " * " + ((String) commentlist.get(count)).trim(), index);
					}

					// Last, we insert the javadoc opening tag.
					file.insert(indent + "/**", index);
				}
				// No star insets.
				else {
					// First, we insert the javadoc comment closing tag.
					file.insert(indent + "*/", index);

					// Then, we insert the list of tags.
					// Go through the array, backwards.
					for(int count = tags.length - 1; count >= 0; count--) {
							// Retrieve the tag and insert it into file.
							file.insert(indent + tags[count].trim(), index);
					}

					// Then, we insert a white line (if there were any tags, that is).
					if(tags.length > 0)  file.insert(indent, index);

					// Then, we insert the comments.
					// Go through commentlist.
					for(int count = 0; count < commentlist.size(); count++) {
						// Retrieve the comment from commentlist, and insert it into
						// file.
						file.insert(indent + ((String) commentlist.get(count)).trim(), index);
					}

					// Then, we insert the javadoc opening tag.
					file.insert(indent + "/**", index);
				}
			}
		}
		// Error!
		catch(WriteNotAllowedException e) {
			// Very. Bad. Very. Stupid.
			// Nothing will be written. Might as well return.
			// Thing is, this shouldn't happen.
			return;
		}
	}


	/**
	 * Adds comment above index with the set javadoc style comment.
	 * Note that anything that is above index will move "upwards" so to speak,
	 * but the line directly above index (the line on index - 1) may not contain
	 * comment, cause comment should be converted then.
	 *
	 * @param index The index of the method/class/attribute that needs comment
	 * converted.
	 * @param tags The tags to be inserted.
	 * @param whitelines The amount of white lines to be inserted.
	 */
	protected void addComment(int index, String[] tags, int whitelines) {
		// Catch errors.
		try {
			// Get the indentation for the comment.
			String indent = getIndentation(index);

			// Check whether the line above the current line is not comment.
			// Also allowed is the fact that the current line is the start of the
			// file.
			if(index == 0 || (!file.get(index - 1).trim().startsWith("//") && !file.get(index - 1).trim().startsWith("*/"))) {
				// No comment there. Add the comment!
				// To do this we go through tags backwards and insert them one by
				// one. Then we insert the required number of white lines.

				// Check which javadoc style is required.
				if(starinsets) {
					// First, we insert the javadoc comment closing tag.
					file.insert(indent + " */", index);

					// Then, we insert the list of tags.
					// Go through the array, backwards.
					for(int count = tags.length - 1; count >= 0; count--) {
						// Retrieve the tag and insert it into file, with the javadoc
						// star in front of it.
						file.insert(indent + " * " + tags[count].trim(), index);
					}

					// Then, we insert the white lines.
					for(int count = 0; count < whitelines; count++) {
						// Insert a white line.
						file.insert(indent + " * ", index);
					}

					// Last, we insert the javadoc opening tag.
					file.insert(indent + "/**", index);
				}
				// No star insets.
				else {
					// First, we insert the javadoc comment closing tag.
					file.insert(indent + "*/", index);

					// Then, we insert the list of tags.
					// Go through the array, backwards.
					for(int count = tags.length - 1; count >= 0; count--) {
							// Retrieve the tag and insert it into file.
							file.insert(indent + tags[count].trim(), index);
					}

					// Then, we insert the white lines.
					for(int count = 0; count < whitelines; count++) {
						// Insert a white line.
						file.insert(indent, index);
					}

					// Then, we insert the javadoc opening tag.
					file.insert(indent + "/**", index);
				}
			}
			// The line was comment. We return without having done anything.
			else  return;
		}
		// Error!
		catch(WriteNotAllowedException e) {
			// Very. Bad. Very. Stupid.
			// Nothing will be written. Might as well return.
			// Thing is, this shouldn't happen.
			return;
		}
	}


	/**
	 * Returns the indentation of the line at index as a String.
	 *
	 * @param index The index of the line of which the indentation is to be returned.
	 * @return The indentation of the line at index.
	 */
	protected String getIndentation(int index) {
		// Retrieve the line at index. Construct a StringTokenizer over it. We want
		// the delimiters to be tokenized too. We use the default delimiters, copied
		// straight out of the API documentation.
		// Tokepie, I choose you! Tok! Tok!
		StringTokenizer tokepie = new StringTokenizer(file.get(index), " \t\n\r\f", true);

		// The indentation for all the comment.
		String indent = "";

		// Go through the tokens until a non-whitespace one is encountered.
		// We add up the whitespace to generate the indentation for the comment.
		while(tokepie.hasMoreTokens()) {
			// Retrieve the token.
			String token = tokepie.nextToken();

			// Put all the whitespace tokens in a string.
			if(token.trim().equals(""))  indent += token;
			// A non-whitespace character has been encountered.
			// Return the indentation.
			else  return indent;
		}

		// If we get here then there was only whitespace on that line.
		// Return the indentation.
		return indent;
	}


	/**
	 * The FileLineList that is used for editing the file.
	 */
	protected FileLineList file;

	/**
	 * Whether or not this CommentConverter uses the star insets style of javadoc
	 * commenting.
	 */
	protected boolean starinsets;
}